package com.psol.xbe2;

import java.sql.*;
import java.util.*;

public class MerchantService
{
   public final static String driver = "org.hsql.jdbcDriver",
                              url = "jdbc:HypersonicSQL:db/merchant",
                              user = "sa",
                              password = null;

   public Product[] getProductList()
      throws SQLException
   {
      Vector vector = new Vector();
      try
      {
         Class.forName(driver);
         Connection connection =
            DriverManager.getConnection(url,
                                        user,
                                        password);
         try
         {
            Statement stmt = connection.createStatement();
            try
            {
               ResultSet rs =
                  stmt.executeQuery(
                  "select id, name from products order by name");
               while(rs.next())
               {
                  Product product = new Product();
                  product.setId(rs.getInt(1));
                  product.setName(rs.getString(2));
                  vector.addElement(product);
               }
            }
            finally
            {
                stmt.close();
            }
         }
         finally
         {
            connection.close();
         }
      }
      catch(ClassNotFoundException e)
      {
         e.printStackTrace();
      }
      Product[] productList = new Product[vector.size()];
      vector.copyInto(productList);
      return productList;
   }

   public ProductDetails getProductDetails(int id)
      throws SQLException
   {
      ProductDetails productDetails = null;
      try
      {
         Class.forName(driver);
         Connection connection =
            DriverManager.getConnection(url,
                                        user,
                                        password);
         try
         {
            PreparedStatement stmt =
               connection.prepareStatement("select id, name, " +
                     "manufacturer, warranty, description," +
                     "price from products where id = ?");
            try
            {
               stmt.setInt(1,id);
               ResultSet rs = stmt.executeQuery();
               if(rs.next())
               {
                  productDetails = new ProductDetails();
                  productDetails.setId(rs.getInt(1));
                  productDetails.setName(rs.getString(2));
                  productDetails.setManufacturer(rs.getString(3));
                  productDetails.setWarranty(rs.getString(4));
                  productDetails.setDescription(rs.getString(5));
                  productDetails.setPrice(rs.getDouble(6));
               }
            }
            finally
            {
                stmt.close();
            }
         }
         finally
         {
            connection.close();
         }
      }
      catch(ClassNotFoundException e)
      {
         e.printStackTrace();
      }
      return productDetails;
   }

   public boolean addOrder(String name,
                           String company,
                           String street,
                           String region,
                           String postalCode,
                           String locality,
                           String country,
                           String email,
                           OrderLine[] lines)
   {
      try
      {
         Class.forName(driver);
         Connection connection =
            DriverManager.getConnection(url,
                                        user,
                                        password);
         int maxId = 0;
         try
         {
            Statement stmt = connection.createStatement();
            try
            {
               // not good enough for a live system
               // but portable and safe enough in this context
               ResultSet rs = stmt.executeQuery(
                                 "select max(id) from orders");
               if(rs.next())
                  maxId = rs.getInt(1);
            }
            finally
            {
               stmt.close();
            }
            PreparedStatement insertOrder =
               connection.prepareStatement("insert into orders " +
                     "(id, name, company, street, region, " +
                     "postal_code, locality, country, email) " +
                     "values (?,?,?,?,?,?,?,?,?)");
            try
            {
               insertOrder.setInt(1,maxId + 1);
               insertOrder.setString(2,name);
               insertOrder.setString(3,company);
               insertOrder.setString(4,street);
               insertOrder.setString(5,region);
               insertOrder.setString(6,postalCode);
               insertOrder.setString(7,locality);
               insertOrder.setString(8,country);
               insertOrder.setString(9,email);
               insertOrder.executeUpdate();
            }
            finally
            {
                insertOrder.close();
            }
            PreparedStatement insertLine =
               connection.prepareStatement("insert into lines " +
                     "(orderid, productid, quantity) " +
                     "values(?,?,?)");
            try
            {
               for(int i = 0;i < lines.length;i++)
               {
                  OrderLine line = lines[i];
                  insertLine.setInt(1,maxId + 1);
                  insertLine.setInt(2,line.getId());
                  insertLine.setInt(3,line.getQuantity());
                  insertLine.executeUpdate();
               }
            }
            finally
            {
               insertLine.close();
            }
         }
         finally
         {
            connection.close();
         }
      }
      catch(ClassNotFoundException e)
      {
         System.out.println(e.getMessage());
         return false;
      }
      catch(SQLException e)
      {
         System.out.println(e.getMessage());
         return false;
      }
      catch(Exception e)
      {
         e.printStackTrace();
         return false;
      }
      return true;
   }
}
